// last modified: 10AUG1999: updated for gcc-2.95

#include <string.h>
#include <stdlib.h>
#include <gtk--.h>
#include <gtk/gtk.h>



/*
 *  GuiDialogOAC: create a basic dialog dialog with 3 buttons 
 */
class GuiDialogOAC : public Gtk_Dialog
	{
	public: 
					GuiDialogOAC (char *winTitle=NULL, char *frameTitle=NULL);
					~GuiDialogOAC (); 
		virtual void		buildDialogWindow ();

	protected: 	
		virtual void 		fillVBox ();
		virtual void		fillActionArea ();
		virtual void 		buttonCallbackOK ();
		virtual void 		buttonCallbackApply ();
		virtual void 		buttonCallbackCancel ();
		virtual gint		delete_event_impl (GdkEventAny *e);


		Gtk_Button		d_buttonOK, 
					d_buttonApply, 
					d_buttonCancel;
  		Gtk_Frame		*d_baseFrame;
		char			*d_winTitle,
					*d_frameTitle;

		Gtk_VBox		d_vBoxList,
					d_vBoxMain;
		GSList                  *d_group;
		Gtk_RadioButton         *d_RB1, *d_RB2;
		Gtk_CheckButton         *d_CB;
		int			d_c;
	};



/*
 *  constructor: initialize everything 
 */
GuiDialogOAC::GuiDialogOAC(char *winTitle, char *frameTitle)
	      : Gtk_Dialog (),
		d_buttonOK ("OK"),
		d_buttonApply ("Apply"),
		d_buttonCancel ("Cancel")
{
	if (winTitle)
		d_winTitle = strdup (winTitle);
	else
		d_winTitle = strdup ("GuiDialogOAC Window");

	if (frameTitle)
		d_frameTitle = strdup (frameTitle);
	else
		d_frameTitle = NULL;

	this->set_position (GTK_WIN_POS_MOUSE);		
	d_c=0;
}


/*
 *  destructor: clean up, free allocated memory
 */
GuiDialogOAC::~GuiDialogOAC()
{
	if (d_winTitle) free (d_winTitle);	// c-type alloc from strdup
	if (d_frameTitle) free (d_frameTitle);	// c-type alloc from strdup
	if (d_baseFrame) delete d_baseFrame;
}


/*
 *  buildDialogWindow: 
 */
void GuiDialogOAC::buildDialogWindow ()
{
	this->set_usize (200, 150);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	fillVBox ();
	fillActionArea ();

	this->show ();
}


/*
 *  fillVBox: fill the dialog's VBox 
 */
void GuiDialogOAC::fillVBox ()
{
	if (d_frameTitle)
		{
		d_baseFrame = new Gtk_Frame (d_frameTitle);
		d_baseFrame->set_shadow_type (GTK_SHADOW_ETCHED_IN);
		d_baseFrame->set_border_width (6);
		this->get_vbox()->pack_start (*d_baseFrame, TRUE, TRUE, 0);
		d_baseFrame->show ();	
		}

	d_RB1 = new Gtk_RadioButton ((GSList *)NULL, "Single Flow Direction");
	d_RB1->set_active (TRUE);
	d_vBoxList.pack_start (*d_RB1, TRUE, TRUE, 0);
	d_RB1->show ();

	d_group = d_RB1->group ();
	d_RB2 = new Gtk_RadioButton (d_group, "Multiple Flow Direction");
	d_vBoxList.pack_start (*d_RB2, TRUE, TRUE, 0);
	d_RB2->show ();

	this->d_baseFrame->add (d_vBoxList);

	d_CB = new Gtk_CheckButton ("Observe Sealevel");
	d_CB->set_active (FALSE);
	this->get_vbox()->pack_start (*d_CB, TRUE, TRUE, 0);
	d_CB->show ();

	d_vBoxList.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}


/*
 *  fillActionArea: fill the dialog's action area
 */
void GuiDialogOAC::fillActionArea ()
{
	// create new hbox and add buttons to it
	connect_to_method (d_buttonOK.clicked, this, &GuiDialogOAC::buttonCallbackOK);
	this->get_action_area()->pack_start (d_buttonOK, TRUE, TRUE, 0);
	d_buttonOK.show ();

	connect_to_method (d_buttonApply.clicked, this, &GuiDialogOAC::buttonCallbackApply);
	this->get_action_area()->pack_start (d_buttonApply, TRUE, TRUE, 0);
	d_buttonApply.show ();

	connect_to_method (d_buttonCancel.clicked, this, &GuiDialogOAC::buttonCallbackCancel);
	this->get_action_area()->pack_start (d_buttonCancel, TRUE, TRUE, 0);
	d_buttonCancel.show ();
}


void GuiDialogOAC::buttonCallbackOK ()
{
	char 	*s = new char[80];

	cout << "GuiDialogOAC: Default Callback OK\n";
	this->d_baseFrame->set_label ();
	sprintf (s, "%d", d_c++);
        this->d_baseFrame->set_label (s);
}


void GuiDialogOAC::buttonCallbackApply ()
{
	cout << "GuiDialogOAC: Default Callback Apply\n";
}


void GuiDialogOAC::buttonCallbackCancel ()
{
	exit (0);
}


gint GuiDialogOAC::delete_event_impl(GdkEventAny *)
{
	this->hide ();
	return (0);
}


int main (int argc, char *argv[])
{
	Gtk_Main	gtkMain (&argc, &argv);
	GuiDialogOAC	*testDialog;

	testDialog = new GuiDialogOAC ("GuiDialogOAC", "Test Frame");
	testDialog->buildDialogWindow ();
	testDialog->show ();
	gtkMain.run ();
}
