// test to create a buffered drawing area, based on examples/scribble-simple.cc
// written by Robert Gasch (Robert_Gasch@peoplesoft.com)
// last modified: 10AUG1999: updated for gcc-2.95


#include <gtk--.h>
#include <gtk/gtk.h>


class bufferedDrawingArea : public Gtk_DrawingArea
{
	public:
				bufferedDrawingArea ();

	private: 
		int 		handle_configure_event (GdkEventConfigure *event);
		int		handle_expose_event (GdkEventExpose *event);
		void 		draw_brush (gdouble x, gdouble y);
		gint 		handle_button_press_event (GdkEventButton *event);
		gint 		handle_motion_notify_event (GdkEventMotion *event);

		GdkPixmap 	*pixmap; 	// pixmap buffer for drawing area 
};


class HFWindow : public Gtk_Window
{
	public: 
					HFWindow (); 

	private: 	
		void 			quit ();

		Gtk_VBox 		vbox;
		bufferedDrawingArea 	drawing_area;
		Gtk_Button 		button;
};



bufferedDrawingArea::bufferedDrawingArea () : pixmap (0)
	{
	/* Signals used to handle backing pixmap */
	connect_to_method (expose_event, this, &bufferedDrawingArea::handle_expose_event);
	connect_to_method (configure_event, this, &bufferedDrawingArea::handle_configure_event);

	/* Event signals */
	connect_to_method (motion_notify_event, this, &bufferedDrawingArea::handle_motion_notify_event);
	connect_to_method (button_press_event, this, &bufferedDrawingArea::handle_button_press_event);

	set_events (GDK_EXPOSURE_MASK
		| GDK_LEAVE_NOTIFY_MASK
		| GDK_BUTTON_PRESS_MASK
		| GDK_POINTER_MOTION_MASK
		| GDK_POINTER_MOTION_HINT_MASK);
	}


/* Create a new backing pixmap of the appropriate size */
int bufferedDrawingArea::handle_configure_event (GdkEventConfigure *event)
	{
    	//GtkWidget 	*widget = GTK_WIDGET(gtkobj());

	if (pixmap)
		gdk_pixmap_unref(pixmap);

	pixmap = gdk_pixmap_new(get_window(),  width(), height(), -1);
    	gdk_draw_rectangle (pixmap, 
		GTK_WIDGET(gtkobj())->style->white_gc,
		TRUE, 0, 0, width(), height());

	return TRUE;
	}


/* Redraw the screen from the backing pixmap */
int bufferedDrawingArea::handle_expose_event (GdkEventExpose *event)
	{
	gdk_draw_pixmap(get_window(),
	    GTK_WIDGET (gtkobj())->style->fg_gc[GTK_WIDGET_STATE (GTK_WIDGET(gtkobj()))],
	    pixmap,
	    event->area.x, event->area.y,
	    event->area.x, event->area.y,
	    event->area.width, event->area.height);
	return FALSE;
	}


/* Draw a rectangle on the screen */
void bufferedDrawingArea::draw_brush (gdouble x, gdouble y)
	{
	GdkRectangle update_rect;
	update_rect.x = (int)x - 5;
	update_rect.y = (int)y - 5;
	update_rect.width = 10;
	update_rect.height = 10;
	gdk_draw_rectangle (pixmap,
		GTK_WIDGET (gtkobj())->style->black_gc,
		TRUE,
		update_rect.x, update_rect.y,
		update_rect.width, update_rect.height);

	unsigned char	buf[10*3];
	for (int i=0; i<30; i++)
		buf[i] = 0;
	for (int i=0; i<10; i++)
		gdk_draw_rgb_image (pixmap,
			GTK_WIDGET (gtkobj())->style->black_gc,
			update_rect.x+20, update_rect.y+20, 
			10, 1, GDK_RGB_DITHER_NONE,
			buf, 10);
			
	update_rect.x = 0;
	update_rect.y = 0;
	update_rect.width = 400;
	update_rect.height = 400;

	draw (&update_rect);
	}


gint bufferedDrawingArea::handle_button_press_event (GdkEventButton *event)
	{
	if (event->button == 1 && pixmap != NULL)
		draw_brush (event->x, event->y);

	return TRUE;
	}


gint bufferedDrawingArea::handle_motion_notify_event (GdkEventMotion *event)
	{
	int 		x, y;
	GdkModifierType state;

	if (event->is_hint)
		gdk_window_get_pointer (event->window, &x, &y, &state);
	else
		{
		x = (int)event->x;
		y = (int)event->y;
		state = (GdkModifierType) event->state;
		}
    
	if (state & GDK_BUTTON1_MASK && pixmap != NULL)
		draw_brush (x, y);

	return TRUE;
	}



HFWindow::HFWindow() : Gtk_Window(GTK_WINDOW_TOPLEVEL),
       			vbox (FALSE, 0),
       			button ("quit")
	{
	add (&vbox);

	/* Create the drawing area */
	drawing_area.size (400, 400);
	vbox.pack_start (drawing_area, TRUE, TRUE, 0);

	/* Add the button */
	vbox.pack_start (button, FALSE, FALSE, 0);

	connect_to_method (button.clicked, this, &HFWindow::quit);
	connect_to_method (destroy, this, &HFWindow::quit);

	drawing_area.show();
	button.show();
	vbox.show();
	}


void HFWindow::quit ()
	{
	gtk_exit (0);
	}


int main (int argc, char *argv[])
{
	HFWindow 	*hfw;
	Gtk_Main 	hfedit (&argc, &argv);

	gdk_rgb_init ();
	hfw = new HFWindow;
	hfw ->show();

	hfedit.run();

	return 0;
}
